PART_OF_SPEACH = {
	'adj': 'adjective',
	'adp': 'adposition',
	'adv': 'adverb',
	'cconj': 'coordinating_conjunction',
	'det': 'determiner',
	'propn': 'proper_noun',
	'punct': 'punctuation',
	'num': 'numeric_modifier',
	'sym': 'symbol',
	'pron':'pronoun'
}

